@@ -18,38 +18,39 @@
 #ifdef QT_DBUS_LIB
 #include "kmainwindowiface_p.h"
 #endif
-#include "ktoolbarhandler_p.h"
+#include "ktooltiphelper.h"
 #include "khelpmenu.h"
 #include "ktoolbar.h"
+#include "ktoolbarhandler_p.h"
 
 #include <QApplication>
-#include <QList>
-#include <QObject>
-#include <QTimer>
 #include <QCloseEvent>
 #include <QDockWidget>
+#include <QFile>
+#include <QList>
 #include <QMenuBar>
+#include <QObject>
 #include <QSessionManager>
 #include <QStatusBar>
 #include <QStyle>
+#include <QTimer>
 #include <QWidget>
 #include <QWindow>
 #ifdef QT_DBUS_LIB
 #include <QDBusConnection>
 #endif
 
-#include <ktoggleaction.h>
-#include <kaboutdata.h>
-#include <kconfig.h>
-#include <ksharedconfig.h>
-#include <klocalizedstring.h>
-#include <kconfiggroup.h>
-#include <kwindowconfig.h>
-#include <kconfiggui.h>
+#include <KAboutData>
+#include <KConfig>
+#include <KConfigGroup>
+#include <KConfigGui>
+#include <KLocalizedString>
+#include <KSharedConfig>
+#include <KWindowConfig>
 
-//#include <ctype.h>
+//#include <cctype>
 
-static const char WINDOW_PROPERTIES[]="WindowProperties";
+static const char WINDOW_PROPERTIES[] = "WindowProperties";
 
 static QMenuBar *internalMenuBar(KMainWindow *mw)
 {
@@ -81,9 +82,9 @@ private:
     KMainWindow *const m_win;
 };
 
-DockResizeListener::DockResizeListener(KMainWindow *win) :
-    QObject(win),
-    m_win(win)
+DockResizeListener::DockResizeListener(KMainWindow *win)
+    : QObject(win)
+    , m_win(win)
 {
 }
 
@@ -110,10 +111,8 @@ bool DockResizeListener::eventFilter(QObject *watched, QEvent *event)
 
 KMWSessionManager::KMWSessionManager()
 {
-    connect(qApp, &QGuiApplication::saveStateRequest,
-            this, &KMWSessionManager::saveState);
-    connect(qApp, &QGuiApplication::commitDataRequest,
-            this, &KMWSessionManager::commitData);
+    connect(qApp, &QGuiApplication::saveStateRequest, this, &KMWSessionManager::saveState);
+    connect(qApp, &QGuiApplication::commitDataRequest, this, &KMWSessionManager::commitData);
 }
 
 KMWSessionManager::~KMWSessionManager()
@@ -145,7 +144,7 @@ void KMWSessionManager::saveState(QSessionManager &sm)
     config->sync();
 
     // generate discard command for new file
-    QString localFilePath =  QStandardPaths::writableLocation(QStandardPaths::GenericConfigLocation) + QLatin1Char('/') + config->name();
+    QString localFilePath = QStandardPaths::writableLocation(QStandardPaths::GenericConfigLocation) + QLatin1Char('/') + config->name();
     if (QFile::exists(localFilePath)) {
         QStringList discard;
         discard << QStringLiteral("rm");
@@ -195,15 +194,21 @@ Q_GLOBAL_STATIC(KMWSessionManager, ksm)
 Q_GLOBAL_STATIC(QList<KMainWindow *>, sMemberList)
 
 KMainWindow::KMainWindow(QWidget *parent, Qt::WindowFlags f)
-    : QMainWindow(parent, f), k_ptr(new KMainWindowPrivate)
+    : QMainWindow(parent, f)
+    , k_ptr(new KMainWindowPrivate)
 {
-    k_ptr->init(this);
+    Q_D(KMainWindow);
+
+    d->init(this);
 }
 
 KMainWindow::KMainWindow(KMainWindowPrivate &dd, QWidget *parent, Qt::WindowFlags f)
-    : QMainWindow(parent, f), k_ptr(&dd)
+    : QMainWindow(parent, f)
+    , k_ptr(&dd)
 {
-    k_ptr->init(this);
+    Q_D(KMainWindow);
+
+    d->init(this);
 }
 
 void KMainWindowPrivate::init(KMainWindow *_q)
@@ -216,7 +221,7 @@ void KMainWindowPrivate::init(KMainWindow *_q)
 
     helpMenu = nullptr;
 
-    //actionCollection()->setWidget( this );
+    // actionCollection()->setWidget( this );
 #if 0
     QObject::connect(KGlobalSettings::self(), SIGNAL(settingsChanged(int)),
                      q, SLOT(_k_slotSettingsChanged(int)));
@@ -248,9 +253,8 @@ void KMainWindowPrivate::init(KMainWindow *_q)
     // application's own message catalog instead of kxmlgui's.
     KAboutData aboutData(KAboutData::applicationData());
     if (aboutData.translators().isEmpty()) {
-        aboutData.setTranslator(
-                i18ndc(nullptr, "NAME OF TRANSLATORS", "Your names"),
-                i18ndc(nullptr, "EMAIL OF TRANSLATORS", "Your emails"));
+        aboutData.setTranslator(i18ndc(nullptr, "NAME OF TRANSLATORS", "Your names"), //
+                                i18ndc(nullptr, "EMAIL OF TRANSLATORS", "Your emails"));
 
         KAboutData::setApplicationData(aboutData);
     }
@@ -258,7 +262,7 @@ void KMainWindowPrivate::init(KMainWindow *_q)
     settingsDirty = false;
     autoSaveSettings = false;
     autoSaveWindowSize = true; // for compatibility
-    //d->kaccel = actionCollection()->kaccel();
+    // d->kaccel = actionCollection()->kaccel();
     settingsTimer = nullptr;
     sizeTimer = nullptr;
 
@@ -267,17 +271,17 @@ void KMainWindowPrivate::init(KMainWindow *_q)
 
     sizeApplied = false;
     suppressCloseEvent = false;
+
+    qApp->installEventFilter(KToolTipHelper::instance());
 }
 
 static bool endsWithHashNumber(const QString &s)
 {
-    for (int i = s.length() - 1;
-            i > 0;
-            --i) {
-        if (s[ i ] == QLatin1Char('#') && i != s.length() - 1) {
-            return true;    // ok
+    for (int i = s.length() - 1; i > 0; --i) {
+        if (s[i] == QLatin1Char('#') && i != s.length() - 1) {
+            return true; // ok
         }
-        if (!s[ i ].isDigit()) {
+        if (!s[i].isDigit()) {
             break;
         }
     }
@@ -287,10 +291,12 @@ static bool endsWithHashNumber(const QString &s)
 static inline bool isValidDBusObjectPathCharacter(const QChar &c)
 {
     ushort u = c.unicode();
+    /* clang-format off */
     return (u >= QLatin1Char('a') && u <= QLatin1Char('z'))
-           || (u >= QLatin1Char('A') && u <= QLatin1Char('Z'))
-           || (u >= QLatin1Char('0') && u <= QLatin1Char('9'))
-           || (u == QLatin1Char('_')) || (u == QLatin1Char('/'));
+        || (u >= QLatin1Char('A') && u <= QLatin1Char('Z'))
+        || (u >= QLatin1Char('0') && u <= QLatin1Char('9'))
+        || (u == QLatin1Char('_')) || (u == QLatin1Char('/'));
+    /* clang-format off */
 }
 
 void KMainWindowPrivate::polish(KMainWindow *q)
@@ -336,7 +342,7 @@ void KMainWindowPrivate::polish(KMainWindow *q)
             break;
         }
         if (tryReuse) {
-            objname = name.left(name.length() - 1);   // lose the hash
+            objname = name.left(name.length() - 1); // lose the hash
             unusedNumber = 0; // start from 1 below
             tryReuse = false;
         }
@@ -345,8 +351,8 @@ void KMainWindowPrivate::polish(KMainWindow *q)
     }
     q->setObjectName(s);
     if (!q->window() || q->window() == q) {
-         q->winId(); // workaround for setWindowRole() crashing, and set also window role, just in case TT
-         q->setWindowRole(s);   // will keep insisting that object name suddenly should not be used for window role
+        q->winId(); // workaround for setWindowRole() crashing, and set also window role, just in case TT
+        q->setWindowRole(s); // will keep insisting that object name suddenly should not be used for window role
     }
 
     dbusName = QLatin1Char('/') + QCoreApplication::applicationName() + QLatin1Char('/');
@@ -359,11 +365,14 @@ void KMainWindowPrivate::polish(KMainWindow *q)
     }
 
 #ifdef QT_DBUS_LIB
-    QDBusConnection::sessionBus().registerObject(dbusName, q, QDBusConnection::ExportScriptableSlots |
-            QDBusConnection::ExportScriptableProperties |
-            QDBusConnection::ExportNonScriptableSlots |
-            QDBusConnection::ExportNonScriptableProperties |
-            QDBusConnection::ExportAdaptors);
+    /* clang-format off */
+    constexpr auto opts = QDBusConnection::ExportScriptableSlots
+                          | QDBusConnection::ExportScriptableProperties
+                          | QDBusConnection::ExportNonScriptableSlots
+                          | QDBusConnection::ExportNonScriptableProperties
+                          | QDBusConnection::ExportAdaptors;
+    /* clang-format on */
+    QDBusConnection::sessionBus().registerObject(dbusName, q, opts);
 #endif
 }
 
@@ -380,8 +389,7 @@ void KMainWindowPrivate::setSettingsDirty(CallCompression callCompression)
                 settingsTimer = new QTimer(q);
                 settingsTimer->setInterval(500);
                 settingsTimer->setSingleShot(true);
-                QObject::connect(settingsTimer, &QTimer::timeout,
-                                 q, &KMainWindow::saveAutoSaveSettings);
+                QObject::connect(settingsTimer, &QTimer::timeout, q, &KMainWindow::saveAutoSaveSettings);
             }
             settingsTimer->start();
         } else {
@@ -397,7 +405,9 @@ void KMainWindowPrivate::setSizeDirty()
             sizeTimer = new QTimer(q);
             sizeTimer->setInterval(500);
             sizeTimer->setSingleShot(true);
-            QObject::connect(sizeTimer, SIGNAL(timeout()), q, SLOT(_k_slotSaveAutoSaveSize()));
+            QObject::connect(sizeTimer, &QTimer::timeout, q, [this]() {
+                _k_slotSaveAutoSaveSize();
+            });
         }
         sizeTimer->start();
     }
@@ -406,14 +416,13 @@ void KMainWindowPrivate::setSizeDirty()
 KMainWindow::~KMainWindow()
 {
     sMemberList()->removeAll(this);
-    delete static_cast<QObject *>(k_ptr->dockResizeListener);  //so we don't get anymore events after k_ptr is destroyed
-    delete k_ptr;
+    delete static_cast<QObject *>(k_ptr->dockResizeListener); // so we don't get anymore events after k_ptr is destroyed
 }
 
 #if KXMLGUI_BUILD_DEPRECATED_SINCE(5, 0)
 QMenu *KMainWindow::helpMenu(const QString &aboutAppText, bool showWhatsThis)
 {
-    K_D(KMainWindow);
+    Q_D(KMainWindow);
     if (!d->helpMenu) {
         if (aboutAppText.isEmpty()) {
             d->helpMenu = new KHelpMenu(this, KAboutData::applicationData(), showWhatsThis);
@@ -431,11 +440,10 @@ QMenu *KMainWindow::helpMenu(const QString &aboutAppText, bool showWhatsThis)
 
 QMenu *KMainWindow::customHelpMenu(bool showWhatsThis)
 {
-    K_D(KMainWindow);
+    Q_D(KMainWindow);
     if (!d->helpMenu) {
         d->helpMenu = new KHelpMenu(this, QString(), showWhatsThis);
-        connect(d->helpMenu, &KHelpMenu::showAboutApplication,
-                this, &KMainWindow::showAboutApplication);
+        connect(d->helpMenu, &KHelpMenu::showAboutApplication, this, &KMainWindow::showAboutApplication);
     }
 
     return d->helpMenu->menu();
@@ -508,7 +516,7 @@ void KMainWindow::setPlainCaption(const QString &caption)
 
 void KMainWindow::appHelpActivated()
 {
-    K_D(KMainWindow);
+    Q_D(KMainWindow);
     if (!d->helpMenu) {
         d->helpMenu = new KHelpMenu(this);
         if (!d->helpMenu) {
@@ -520,7 +528,7 @@ void KMainWindow::appHelpActivated()
 
 void KMainWindow::closeEvent(QCloseEvent *e)
 {
-    K_D(KMainWindow);
+    Q_D(KMainWindow);
     if (d->suppressCloseEvent) {
         e->accept();
         return;
@@ -549,7 +557,7 @@ void KMainWindow::closeEvent(QCloseEvent *e)
         d->letDirtySettings = false;
         e->accept();
     } else {
-        e->ignore();    //if the window should not be closed, don't close it
+        e->ignore(); // if the window should not be closed, don't close it
     }
     // If saving session, we are processing a fake close event, and might get the real one later.
     if (e->isAccepted() && qApp->isSavingSession())
@@ -571,7 +579,7 @@ void KMainWindow::readGlobalProperties(KConfig *)
 
 void KMainWindow::savePropertiesInternal(KConfig *config, int number)
 {
-    K_D(KMainWindow);
+    Q_D(KMainWindow);
     const bool oldASWS = d->autoSaveWindowSize;
     d->autoSaveWindowSize = true; // make saveMainWindowSettings save the window size
 
@@ -592,8 +600,8 @@ void KMainWindow::savePropertiesInternal(KConfig *config, int number)
 
 void KMainWindow::saveMainWindowSettings(KConfigGroup &cg)
 {
-    K_D(KMainWindow);
-    //qDebug(200) << "KMainWindow::saveMainWindowSettings " << cg.name();
+    Q_D(KMainWindow);
+    // qDebug(200) << "KMainWindow::saveMainWindowSettings " << cg.name();
 
     // Called by session management - or if we want to save the window size anyway
     if (d->autoSaveWindowSize) {
@@ -649,7 +657,7 @@ void KMainWindow::saveMainWindowSettings(KConfigGroup &cg)
 
 bool KMainWindow::readPropertiesInternal(KConfig *config, int number)
 {
-    K_D(KMainWindow);
+    Q_D(KMainWindow);
 
     const bool oldLetDirtySettings = d->letDirtySettings;
     d->letDirtySettings = false;
@@ -680,8 +688,8 @@ bool KMainWindow::readPropertiesInternal(KConfig *config, int number)
 
 void KMainWindow::applyMainWindowSettings(const KConfigGroup &cg)
 {
-    K_D(KMainWindow);
-    //qDebug(200) << "KMainWindow::applyMainWindowSettings " << cg.name();
+    Q_D(KMainWindow);
+    // qDebug(200) << "KMainWindow::applyMainWindowSettings " << cg.name();
 
     QWidget *focusedWidget = QApplication::focusWidget();
 
@@ -690,6 +698,12 @@ void KMainWindow::applyMainWindowSettings(const KConfigGroup &cg)
 
     if (!d->sizeApplied && (!window() || window() == this)) {
         winId(); // ensure there's a window created
+        // Set the window's size from the existing widget geometry to respect the
+        // implicit size when there is no saved geometry in the config file for
+        // KWindowConfig::restoreWindowSize() to restore
+        // TODO: remove once QTBUG-40584 is fixed; see below
+        windowHandle()->setWidth(width());
+        windowHandle()->setHeight(height());
         KWindowConfig::restoreWindowSize(windowHandle(), cg);
         // NOTICE: QWindow::setGeometry() does NOT impact the backing QWidget geometry even if the platform
         // window was created -> QTBUG-40584. We therefore copy the size here.
@@ -716,16 +730,16 @@ void KMainWindow::applyMainWindowSettings(const KConfigGroup &cg)
     QStatusBar *sb = internalStatusBar(this);
     if (sb) {
         QString entry = cg.readEntry("StatusBar", "Enabled");
-        sb->setVisible( entry != QLatin1String("Disabled") );
+        sb->setVisible(entry != QLatin1String("Disabled"));
     }
 
     QMenuBar *mb = internalMenuBar(this);
     if (mb) {
         QString entry = cg.readEntry("MenuBar", "Enabled");
-        mb->setVisible( entry != QLatin1String("Disabled") );
+        mb->setVisible(entry != QLatin1String("Disabled"));
     }
 
-    if (!autoSaveSettings() || cg.name() == autoSaveGroup()) {   // TODO should be cg == d->autoSaveGroup, to compare both kconfig and group name
+    if (!autoSaveSettings() || cg.name() == autoSaveGroup()) { // TODO should be cg == d->autoSaveGroup, to compare both kconfig and group name
         QString entry = cg.readEntry("ToolBarsMovable", "Disabled");
         KToolBar::setToolBarsLocked(entry == QLatin1String("Disabled"));
     }
@@ -775,13 +789,13 @@ void KMainWindow::saveWindowSize(KConfigGroup &cg) const
 
 void KMainWindow::setSettingsDirty()
 {
-    K_D(KMainWindow);
+    Q_D(KMainWindow);
     d->setSettingsDirty();
 }
 
 bool KMainWindow::settingsDirty() const
 {
-    K_D(const KMainWindow);
+    Q_D(const KMainWindow);
     return d->settingsDirty;
 }
 
@@ -790,14 +804,13 @@ void KMainWindow::setAutoSaveSettings(const QString &groupName, bool saveWindowS
     setAutoSaveSettings(KConfigGroup(KSharedConfig::openConfig(), groupName), saveWindowSize);
 }
 
-void KMainWindow::setAutoSaveSettings(const KConfigGroup &group,
-                                      bool saveWindowSize)
+void KMainWindow::setAutoSaveSettings(const KConfigGroup &group, bool saveWindowSize)
 {
     // We re making a little assumption that if you want to save the window
     // size, you probably also want to save the window position too
     // This avoids having to re-implement a new version of
     // KMainWindow::setAutoSaveSettings that handles these cases independently
-    K_D(KMainWindow);
+    Q_D(KMainWindow);
     d->autoSaveSettings = true;
     d->autoSaveGroup = group;
     d->autoSaveWindowSize = saveWindowSize;
@@ -812,7 +825,7 @@ void KMainWindow::setAutoSaveSettings(const KConfigGroup &group,
 
 void KMainWindow::resetAutoSaveSettings()
 {
-    K_D(KMainWindow);
+    Q_D(KMainWindow);
     d->autoSaveSettings = false;
     if (d->settingsTimer) {
         d->settingsTimer->stop();
@@ -821,27 +834,27 @@ void KMainWindow::resetAutoSaveSettings()
 
 bool KMainWindow::autoSaveSettings() const
 {
-    K_D(const KMainWindow);
+    Q_D(const KMainWindow);
     return d->autoSaveSettings;
 }
 
 QString KMainWindow::autoSaveGroup() const
 {
-    K_D(const KMainWindow);
+    Q_D(const KMainWindow);
     return d->autoSaveSettings ? d->autoSaveGroup.name() : QString();
 }
 
 KConfigGroup KMainWindow::autoSaveConfigGroup() const
 {
-    K_D(const KMainWindow);
+    Q_D(const KMainWindow);
     return d->autoSaveSettings ? d->autoSaveGroup : KConfigGroup();
 }
 
 void KMainWindow::saveAutoSaveSettings()
 {
-    K_D(KMainWindow);
+    Q_D(KMainWindow);
     Q_ASSERT(d->autoSaveSettings);
-    //qDebug(200) << "KMainWindow::saveAutoSaveSettings -> saving settings";
+    // qDebug(200) << "KMainWindow::saveAutoSaveSettings -> saving settings";
     saveMainWindowSettings(d->autoSaveGroup);
     d->autoSaveGroup.sync();
     d->settingsDirty = false;
@@ -849,7 +862,7 @@ void KMainWindow::saveAutoSaveSettings()
 
 bool KMainWindow::event(QEvent *ev)
 {
-    K_D(KMainWindow);
+    Q_D(KMainWindow);
     switch (ev->type()) {
 #if defined(Q_OS_WIN) || defined(Q_OS_OSX)
     case QEvent::Move:
@@ -866,43 +879,39 @@ bool KMainWindow::event(QEvent *ev)
         KToolBar *toolbar = qobject_cast<KToolBar *>(event->child());
         QMenuBar *menubar = qobject_cast<QMenuBar *>(event->child());
         if (dock) {
-            connect(dock, &QDockWidget::dockLocationChanged,
-                    this, &KMainWindow::setSettingsDirty);
-            connect(dock, &QDockWidget::topLevelChanged,
-                    this, &KMainWindow::setSettingsDirty);
+            connect(dock, &QDockWidget::dockLocationChanged, this, &KMainWindow::setSettingsDirty);
+            connect(dock, &QDockWidget::topLevelChanged, this, &KMainWindow::setSettingsDirty);
 
             // there is no signal emitted if the size of the dock changes,
             // hence install an event filter instead
-            dock->installEventFilter(k_ptr->dockResizeListener);
+            dock->installEventFilter(d->dockResizeListener);
         } else if (toolbar) {
             // there is no signal emitted if the size of the toolbar changes,
             // hence install an event filter instead
-            toolbar->installEventFilter(k_ptr->dockResizeListener);
+            toolbar->installEventFilter(d->dockResizeListener);
         } else if (menubar) {
             // there is no signal emitted if the size of the menubar changes,
             // hence install an event filter instead
-            menubar->installEventFilter(k_ptr->dockResizeListener);
+            menubar->installEventFilter(d->dockResizeListener);
         }
+        break;
     }
-    break;
     case QEvent::ChildRemoved: {
         QChildEvent *event = static_cast<QChildEvent *>(ev);
         QDockWidget *dock = qobject_cast<QDockWidget *>(event->child());
         KToolBar *toolbar = qobject_cast<KToolBar *>(event->child());
         QMenuBar *menubar = qobject_cast<QMenuBar *>(event->child());
         if (dock) {
-            disconnect(dock, &QDockWidget::dockLocationChanged,
-                       this, &KMainWindow::setSettingsDirty);
-            disconnect(dock, &QDockWidget::topLevelChanged,
-                       this, &KMainWindow::setSettingsDirty);
-            dock->removeEventFilter(k_ptr->dockResizeListener);
+            disconnect(dock, &QDockWidget::dockLocationChanged, this, &KMainWindow::setSettingsDirty);
+            disconnect(dock, &QDockWidget::topLevelChanged, this, &KMainWindow::setSettingsDirty);
+            dock->removeEventFilter(d->dockResizeListener);
         } else if (toolbar) {
-            toolbar->removeEventFilter(k_ptr->dockResizeListener);
+            toolbar->removeEventFilter(d->dockResizeListener);
         } else if (menubar) {
-            menubar->removeEventFilter(k_ptr->dockResizeListener);
+            menubar->removeEventFilter(d->dockResizeListener);
         }
+        break;
     }
-    break;
     default:
         break;
     }
@@ -977,8 +986,10 @@ QList<KMainWindow *> KMainWindow::memberList()
 
 QString KMainWindow::dbusName() const
 {
-    return k_func()->dbusName;
+    Q_D(const KMainWindow);
+
+    return d->dbusName;
 }
 
-#include "moc_kmainwindow.cpp"
 #include "kmainwindow.moc"
+#include "moc_kmainwindow.cpp"
